@@ -145,7 +145,7 @@ def get_page_encoding(page):
 
 def get_page_base(page, html):
     base = html.find("base")
-    if base:
+    if base and base.has_key("href"):
         return base["href"]
     return page.geturl()
 
@@ -162,32 +162,15 @@ def get_feed_title(xml):
     return "untitled"
 
 
-def discover_feed(url):
-    """ Returns RSS or Atom feed url from related html page.
-
-    Not only RSS or Atom feed urls may be entered by user, so
-    this function check what is passed and if it is html page
-    function try to detect feed url. It also retrieves icon.
-    """
-    page = urlopener.open(url)
-    icon_url = None
-    content_type = None
-    if page.info().has_key("Content-Type"):
-        content_type = page.info()["Content-Type"]
-    if content_type and content_type.find("html") >=0:
-        html = BeautifulSoup(page, fromEncoding = get_page_encoding(page))
-        base = get_page_base(page, html)
-        icon_tag = html.find(name = "link", attrs = {"rel": re.compile("icon", flags = re.I)})
-        if icon_tag:
-            icon_url = urljoin(base, icon_tag["href"])
-        rss_tag = html.find(name = "link", attrs = {"rel": "alternate", "type": "application/rss+xml"})
-        if not rss_tag:
-            rss_tag = html.find(name = "link", attrs = {"rel": "alternate", "type": "application/atom+xml"})
-        if rss_tag:
-            url = urljoin(base, rss_tag["href"])
+def get_feed_details(url, icon_url = None, page = None):
+    if not page:
+        try:
             page = urlopener.open(url)
-            content_type = page.info()["Content-Type"]
-    xml = BeautifulStoneSoup(page, markupMassage = massage)
+            xml = BeautifulStoneSoup(page, markupMassage = massage)
+        finally:
+            page.close()
+    else:
+        xml = BeautifulStoneSoup(page, markupMassage = massage)
     title = get_feed_title(xml)
     icon_tag = xml.find(["icon", "logo", "image"])
     if icon_tag:
@@ -205,13 +188,54 @@ def discover_feed(url):
             if html_tag:
                 html_url = html_tag.string
         if html_url:
-            page = urlopener.open(html_url)
-            html = BeautifulSoup(page, fromEncoding = get_page_encoding(page))
-            base = get_page_base(page, html)
+            html_page = urlopener.open(html_url)
+            try:
+                html = BeautifulSoup(html_page, fromEncoding = get_page_encoding(html_page))
+                base = get_page_base(html_page, html)
+            finally:
+                html_page.close()
             icon_tag = html.find(name = "link", attrs = {"rel": re.compile("icon")})
             if icon_tag:
                 icon_url = urljoin(base, icon_tag["href"])
-    return (url, title, icon_url)
+    icon_filename = ""
+    icon_data = None
+    if icon_url:
+        icon_filename = urlparse(icon_url)[2].split("/")[-1]
+        icon = urlopener.open(icon_url)
+        try:
+            icon_data = icon.read()
+        finally:
+            icon.close()
+    return (url, title, icon_filename, icon_data)
+
+
+def discover_feed(url):
+    """ Returns RSS or Atom feed url from related html page.
+
+    Not only RSS or Atom feed urls may be entered by user, so
+    this function check what is passed and if it is html page
+    function try to detect feed url. It also retrieves icon.
+    """
+    page = urlopener.open(url)
+    try:
+        icon_url = None
+        content_type = None
+        if page.info().has_key("Content-Type"):
+            content_type = page.info()["Content-Type"]
+        if content_type and content_type.find("html") >=0:
+            html = BeautifulSoup(page, fromEncoding = get_page_encoding(page))
+            base = get_page_base(page, html)
+            icon_tag = html.find(name = "link", attrs = {"rel": re.compile("icon", flags = re.I)})
+            if icon_tag:
+                icon_url = urljoin(base, icon_tag["href"])
+            rss_tag = html.find(name = "link", attrs = {"rel": "alternate", "type": "application/rss+xml"})
+            if not rss_tag:
+                rss_tag = html.find(name = "link", attrs = {"rel": "alternate", "type": "application/atom+xml"})
+            if rss_tag:
+                return get_feed_details(urljoin(base, rss_tag["href"]), icon_url)
+        return get_feed_details(url, icon_url, page)
+    finally:
+        page.close()
 
 
 class Downloader:
@@ -354,10 +378,14 @@ class Item:
         """
         self.__report("downloading", url)
         page = urlopener.open(urllib2.Request(url=quote_url_path(url), **kwargs))
-        self.__report("parsing", url)
-        html = BeautifulSoup(page, fromEncoding = get_page_encoding(page))
-        base = get_page_base(page, html)
-        return (html, base, page.geturl())
+        try:
+            self.__report("parsing", url)
+            html = BeautifulSoup(page, fromEncoding = get_page_encoding(page))
+            base = get_page_base(page, html)
+            url = page.geturl()
+        finally:
+            page.close()
+        return (html, base, url)
 
 
     def __cache_next_page(self, html, base):
@@ -739,9 +767,12 @@ class Feed:
         try:
             self.__report("downloading", self.__url)
             page = urlopener.open(quote_url_path(self.__url))
-            self._real_url = page.geturl()
-            self.__report("parsing", self.__url)
-            xml = BeautifulStoneSoup(page, markupMassage = massage)
+            try:
+                self._real_url = page.geturl()
+                self.__report("parsing", self.__url)
+                xml = BeautifulStoneSoup(page, markupMassage = massage)
+            finally:
+                page.close()
             if not self.__title:
                 self.__title = get_feed_title(xml)
             path = os.path.join(self.__path, self.__title + ".html")
