@@ -12,20 +12,20 @@ import ConfigParser
 from datetime import date
 from datetime import timedelta
 from datetime import datetime
-import urlparse
 import feedcircuit
 import urllib
 import webbrowser
 import glob
 import shutil
 import sys
+from BeautifulSoup import BeautifulStoneSoup
 
 gtk.gdk.threads_init()
 
 #This is to be able to start Feed Circuit on PC
 try:
     import osso
-    osso_context = osso.Context("org.maemo.feedcircuit", "0.6.8", False)
+    osso_context = osso.Context("org.maemo.feedcircuit", "0.6.9", False)
 except ImportError:
     pass
 
@@ -70,34 +70,46 @@ def create_config_dir():
     return path
 
 
-class DiscoveryThread(Thread):
+def save_icon(filename, icon_data):
+    f = open(filename, "w")
+    try:
+        f.write(icon_data)
+    finally:
+        f.close()
 
 
-    def __init__(self, url):
+class BannerThread(Thread):
+
+
+    def __init__(self, parent, text):
         Thread.__init__(self)
-        self.url = url
+        self.__banner = hildon.hildon_banner_show_animation(parent, "qgn_indi_pball_a", text)
 
 
     def run(self):
         try:
-            gobject.idle_add(self.done, feedcircuit.discover_feed(self.url), None)
+            result = self.__callback(*self.__args)
+            gobject.idle_add(self.done, result, None)
         except Exception, ex:
             gobject.idle_add(self.done, None, ex)
 
 
     def done(self, result, ex):
-        self.result = result
-        self.exception = ex
+        self.__result = result
+        self.__exception = ex
         gtk.main_quit()
 
-    def discover(self):
-        self.result = None
-        self.exception = None
+    def execute(self, callback, *args):
+        self.__callback = callback
+        self.__args = args
+        self.__result = None
+        self.__exception = None
         self.start()
         gtk.main()
-        if self.exception:
-            raise self.exception
-        return self.result
+        self.__banner.destroy()
+        if self.__exception:
+            raise self.__exception
+        return self.__result
 
 
 class ConfigParserEx(ConfigParser.SafeConfigParser):
@@ -291,9 +303,13 @@ class FeedListModel(gtk.GenericTreeModel):
         return None
 
 
+    def get_config_file_name(self):
+        return os.path.join(self.config_path, "config")
+
+
     def load_config(self):
         parser = ConfigParserEx()
-        parser.read([os.path.join(self.config_path, "config")])
+        parser.read([self.get_config_file_name()])
         self.cache_path = parser.get_default("global", "cache path", self.config_path)
         self.use_checkboxes = parser.getboolean_default("global", "use checkboxes", False)
         for section in parser.sections():
@@ -350,7 +366,6 @@ class FeedListModel(gtk.GenericTreeModel):
             parser.write(f)
         finally:
             f.close()
-        pass
 
 
     def rebuild_map(self):
@@ -461,6 +476,7 @@ class PreferencesWindow(gtk.Dialog):
         texts = [_("Cache path"), _("Show checkboxes")]
         for i in range(0, len(texts)):
             label = gtk.Label(texts[i])
+            label.set_alignment(1, 0.5)
             table.attach(label, 0, 1, i, i + 1, xpadding = 3, ypadding = 3)
         table.attach(hbox, 1, 2, 0, 1, xpadding = 3, ypadding = 3)
         table.attach(self.checkboxes_check, 1, 2, 1, 2, xpadding = 3, ypadding = 3)
@@ -477,6 +493,51 @@ class PreferencesWindow(gtk.Dialog):
         dlg.destroy()
 
 
+class ImportWindow(gtk.Dialog):
+
+
+    def __init__(self, parent):
+        gtk.Dialog.__init__(self, None, parent,
+            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+            (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
+             gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+        self.set_title(_("Import"))
+        hbox = gtk.HBox()
+        self.path_entry = gtk.Entry()
+        self.path_entry.set_width_chars(24)
+        hbox.pack_start(self.path_entry, expand = True)
+        self.choose_button = gtk.ToolButton(stock_id = gtk.STOCK_DIRECTORY)
+        self.choose_button.connect("clicked", self.choose_button_clicked)
+        hbox.pack_start(self.choose_button, expand = False)
+        self.stock_check = gtk.CheckButton()
+        self.stock_check.connect("toggled", self.stock_check_changed)
+        table = gtk.Table(rows = 2, columns = 2)
+        texts = [_("From default reader"), _("OPML file")]
+        for i in range(0, len(texts)):
+            label = gtk.Label(texts[i])
+            label.set_alignment(1, 0.5)
+            table.attach(label, 0, 1, i, i + 1, xpadding = 3, ypadding = 3)
+        table.attach(self.stock_check, 1, 2, 0, 1, xpadding = 3, ypadding = 3)
+        table.attach(hbox, 1, 2, 1, 2, xpadding = 3, ypadding = 3)
+        self.vbox.add(table)
+        self.show_all()
+
+
+    def choose_button_clicked(self, source):
+        dlg = gtk.FileChooserDialog(title = _("Select opml file"), parent = self, action = gtk.FILE_CHOOSER_ACTION_OPEN,
+            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+        if self.path_entry.get_text():
+            dlg.set_filename(os.path.abspath(self.path_entry.get_text()))
+        if dlg.run() == gtk.RESPONSE_ACCEPT:
+            self.path_entry.set_text(dlg.get_filename())
+        dlg.destroy()
+
+
+    def stock_check_changed(self, source):
+        self.path_entry.set_sensitive(self.stock_check.get_active())
+        self.choose_button.set_sensitive(self.stock_check.get_active())
+
+
 class AddWindow(gtk.Dialog):
 
 
@@ -513,7 +574,7 @@ class ExtraWindow(gtk.Dialog):
         texts = [_("Days to keep"), _("Include urls"), _("Exclude urls") ]
         for i in range(0, len(texts)):
             label = gtk.Label(texts[i])
-            label.set_alignment(1, 1)
+            label.set_alignment(1, 0.5)
             table.attach(label, 0, 1, i, i + 1, xpadding = 3, ypadding = 3)
         table.attach(self.days_editor, 1, 2, 0, 1, xpadding = 3, ypadding = 3)
         table.attach(self.include_entry, 1, 2, 1, 2, xpadding = 3, ypadding = 3)
@@ -549,7 +610,7 @@ class ContentWindow(gtk.Dialog):
             _("Ignore printable version"), _("Printable version link"), _("Next page link") ]
         for i in range(0, len(texts)):
             label = gtk.Label(texts[i])
-            label.set_alignment(1, 1)
+            label.set_alignment(1, 0.5)
             table.attach(label, 0, 1, i, i + 1, xpadding = 3, ypadding = 3)
         table.attach(self.as_is_button, 1, 2, 0, 1, xpadding = 3, ypadding = 3)
         table.attach(self.content_tag_entry, 1, 2, 1, 2, xpadding = 3, ypadding = 3)
@@ -622,7 +683,7 @@ class EditWindow(gtk.Dialog):
         texts = [_("Url"), _("Name"), _("Content")]
         for i in range(0, len(texts)):
             label = gtk.Label(texts[i])
-            label.set_alignment(1, 1)
+            label.set_alignment(1, 0.5)
             table.attach(label, 0, 1, i, i + 1, xpadding = 3, ypadding = 3)
         table.attach(hbox, 1, 2, 0, 1, xpadding = 3, ypadding = 3)
         table.attach(self.name_entry, 1, 2, 1, 2, xpadding = 3, ypadding = 3)
@@ -711,28 +772,34 @@ class Program(hildon.Program):
         self.__abort_lock = Lock()
         self.__update_lock = Lock()
         
+        self.__model = FeedListModel(create_config_dir())
+        if not os.path.exists(self.__model.get_config_file_name()):
+            self.import_opml()
+
         menu = gtk.Menu()
         self.__toolbar = gtk.Toolbar()
         self.__toolbar.set_style(gtk.TOOLBAR_ICONS)
         self.append_action(menu, self.__toolbar, None, gtk.STOCK_REFRESH, self.refresh)
         self.append_action(menu, self.__toolbar, None, gtk.STOCK_ADD, self.add)
         menu.append(gtk.SeparatorMenuItem())
-        self.append_action(menu, None, _("Refresh selected"), None, self.refresh_current)
-        self.append_action(menu, None, None, gtk.STOCK_OPEN, self.open)
-        self.append_action(menu, None, None, gtk.STOCK_EDIT, self.edit)
-        self.append_action(menu, None, None, gtk.STOCK_DELETE, self.remove)
+        self.refresh_selected_item = \
+            self.append_action(menu, None, _("Refresh selected"), None, self.refresh_current)
+        self.append_action(menu, self.__toolbar, None, gtk.STOCK_OPEN, self.open)
+        self.append_action(menu, self.__toolbar, None, gtk.STOCK_EDIT, self.edit)
+        self.append_action(menu, self.__toolbar, None, gtk.STOCK_DELETE, self.remove)
         menu.append(gtk.SeparatorMenuItem())
-        self.append_action(menu, None, None, gtk.STOCK_PREFERENCES, self.preferences)
+        self.append_action(menu, None, _("Import"), None, self.import_feeds)
+        self.append_action(menu, self.__toolbar, None, gtk.STOCK_PREFERENCES, self.preferences)
         menu.append(gtk.SeparatorMenuItem())
         self.append_action(menu, None, None, gtk.STOCK_ABOUT, self.about)
         self.append_action(menu, None, None, gtk.STOCK_QUIT, self.stop)
-        self.__progress_label = gtk.Label()
-        self.__progress_label.set_alignment(0, 0.5)
-        self.__progress_label.set_ellipsize(pango.ELLIPSIZE_MIDDLE)
-        status_item = gtk.ToolItem()
-        status_item.add(self.__progress_label)
-        status_item.set_expand(True)
-        self.__toolbar.insert(status_item, -1)
+        #self.__progress_label = gtk.Label()
+        #self.__progress_label.set_alignment(0, 0.5)
+        #self.__progress_label.set_ellipsize(pango.ELLIPSIZE_MIDDLE)
+        #status_item = gtk.ToolItem()
+        #status_item.add(self.__progress_label)
+        #status_item.set_expand(True)
+        #self.__toolbar.insert(status_item, -1)
         self.__abort_button = gtk.ToolButton(gtk.STOCK_STOP)
         self.__abort_button.connect("clicked", self.abort)
         self.__abort_button.set_no_show_all(True)
@@ -740,8 +807,11 @@ class Program(hildon.Program):
 
         self.__window.set_menu(menu)
         
-        self.__feeds = {}
-        self.__model = FeedListModel(create_config_dir())
+        self.refresh_selected_item.set_property("visible",
+            not self.__model.use_checkboxes)
+
+        self.__progress_banner = None
+
         self.__view = gtk.TreeView(self.__model)
         self.__view.set_rubber_banding(True)
         self.__view.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
@@ -760,8 +830,10 @@ class Program(hildon.Program):
         col.pack_start(render_pixbuf, expand=False)
         col.add_attribute(render_pixbuf, 'pixbuf', 0)
         render_text = gtk.CellRendererText()
+        render_text.set_property("ellipsize", pango.ELLIPSIZE_MIDDLE)
         col.pack_start(render_text, expand=True)
         col.add_attribute(render_text, 'markup', 1)
+        col.set_expand(True)
         self.__view.append_column(col)
         
         col = gtk.TreeViewColumn()
@@ -799,6 +871,7 @@ class Program(hildon.Program):
             button = gtk.ToolButton(stock)
             button.connect("clicked", callback)
             toolbar.insert(button, -1)
+        return item
 
 
     def on_key_press(self, widget, event, *args):
@@ -909,12 +982,20 @@ class Program(hildon.Program):
 
 
     def update_complete(self):
-        self.update_status("")
+        #self.update_status("")
+        if self.__progress_banner:
+            self.__progress_banner.destroy()
+            self.__progress_banner = None
         self.__abort_button.hide()
 
 
     def update_status(self, text):
-        self.__progress_label.set_text(text)
+        if self.__progress_banner:
+            self.__progress_banner.set_text(text)
+        else:
+            self.__progress_banner = \
+                hildon.hildon_banner_show_animation(self.__window, "qgn_indi_pball_a", text)
+        #self.__progress_label.set_text(text)
 
 
     def callback(self, item):
@@ -973,7 +1054,12 @@ class Program(hildon.Program):
 
 
     def refresh(self, source):
-        connect_and_exec(self.update_feeds_async, list(self.__model.list))
+        feedlist = None
+        if self.__model.use_checkboxes:
+            feedlist = self.get_selected_feeds()
+        if not feedlist:
+            feedlist = list(self.__model.list)
+        connect_and_exec(self.update_feeds_async, feedlist)
 
 
     def edit(self, source):
@@ -993,26 +1079,21 @@ class Program(hildon.Program):
         if response == gtk.RESPONSE_ACCEPT:
             url = add_window.url_entry.get_text()
             title = ""
-            banner = hildon.hildon_banner_show_animation(self.__window, "qgn_indi_pball_a", _("Discovering feed"))
+            icon_filename = ""
             try:
-                discovery_thread = DiscoveryThread(url)
-                (url, title, icon_url) = discovery_thread.discover()
+                discovery_thread = BannerThread(self.__window, _("Discovering feed"))
+                (url, title, icon_filename, icon_data) = \
+                    discovery_thread.execute(feedcircuit.discover_feed, url)
+                if icon_filename:
+                    icon_filename = uuid.uuid1().hex + icon_filename
+                    save_icon(os.path.join(self.__model.cache_path, icon_filename), icon_data)
             except Exception, ex:
                 msg_dlg = gtk.MessageDialog(parent = self.__window, flags = gtk.DIALOG_MODAL,
                     type=gtk.MESSAGE_WARNING, buttons = gtk.BUTTONS_OK, 
                     message_format = _("Cannot discover feed at given url") + "\n" + str(ex))
                 msg_dlg.run()
                 msg_dlg.destroy()
-            finally:
-                banner.destroy()
-            icon_filename = ""
-            try:
-                if icon_url:
-                    icon_filename = uuid.uuid1().hex + urlparse.urlparse(icon_url)[2].split("/")[-1]
-                    urllib.urlretrieve(icon_url, os.path.join(self.__model.cache_path, icon_filename))
-            except Exception, ex:
-                pass
-            feed_config =FeedConfig(self.__model.cache_path, url, title, FeedConfig.mode_ommit, 7, "", "", False, icon_filename, 0,
+            feed_config = FeedConfig(self.__model.cache_path, url, title, FeedConfig.mode_ommit, 7, "", "", False, icon_filename, 0,
                 False, "", "", "", "")
             edit_window = EditWindow(self.__window, feed_config)
             if edit_window.run() == gtk.RESPONSE_ACCEPT:
@@ -1043,6 +1124,7 @@ class Program(hildon.Program):
             dialog.destroy()
             if response == gtk.RESPONSE_YES:
                 for feed_config in selected:
+                    feed_config.message_count = 0
                     feed = feedcircuit.Feed(None, path = self.__model.cache_path,
                         cache_path = ".cache", filename = feed_config.id + ".html")
                     feed.delete()
@@ -1064,6 +1146,70 @@ class Program(hildon.Program):
         shutil.move(os.path.join(from_path, ".cache"), os.path.join(to_path, ".cache"))
 
 
+    def import_opml(self, filename = None):
+        fav_dir = None
+        if not filename:
+            filename = os.path.expanduser("~/.osso_rss_feed_reader/feedlist.opml")
+            fav_dir = os.path.expanduser("~/.osso_rss_feed_reader/cache/favicons")
+        f = open(filename, "r")
+        try:
+            opml = BeautifulStoneSoup(f)
+            outlines = filter(
+                lambda o: o.has_key("title") and o.has_key("xmlurl"),
+                opml.findAll("outline"))
+            for outline in outlines:
+                icon_file_name = ""
+                if fav_dir:
+                    if outline.has_key("id"):
+                        icon_file = outline["id"] + ".png"
+                        icon_file_src = os.path.join(fav_dir, icon_file)
+                        if os.path.exists(icon_file_src):
+                            icon_file_name = uuid.uuid1().hex + icon_file
+                            shutil.copy(icon_file_src,
+                                os.path.join(self.__model.cache_path, icon_file_name))
+                else:
+                    try:
+                        discovery_thread = BannerThread(self.__window, _("Importing feeds"))
+                        (url, title, icon_file_name, icon_data) = \
+                            discovery_thread.execute(feedcircuit.get_feed_details, outline["xmlurl"])
+                        if icon_file_name:
+                            icon_file_name = uuid.uuid1().hex + icon_file_name
+                            save_icon(os.path.join(self.__model.cache_path, icon_file_name), icon_data)
+                    except Exception, ex:
+                        msg_dlg = gtk.MessageDialog(parent = self.__window, flags = gtk.DIALOG_MODAL,
+                            type=gtk.MESSAGE_WARNING, buttons = gtk.BUTTONS_OK, 
+                            message_format = _("Cannot discover feed at given url") + "\n" + str(ex))
+                        msg_dlg.run()
+                        msg_dlg.destroy()
+                feed_config = FeedConfig(self.__model.cache_path, outline["xmlurl"], outline["title"],
+                    FeedConfig.mode_ommit, 7, "", "", False, icon_file_name, 0,
+                    False, "", "", "", "")
+                self.__model.append(feed_config)
+        finally:
+            f.close()
+
+
+    def import_feeds(self, source):
+        imp_window = ImportWindow(self.__window)
+        try:
+            result = imp_window.run()
+            imp_window.hide()
+            if result == gtk.RESPONSE_ACCEPT:
+                filename = imp_window.path_entry.get_text()
+                if imp_window.stock_check.get_active():
+                    self.import_opml()
+                elif filename:
+                    self.import_opml(filename)
+        except Exception, ex:
+            msg_dlg = gtk.MessageDialog(parent = self.__window, flags = gtk.DIALOG_MODAL,
+                type=gtk.MESSAGE_ERROR, buttons = gtk.BUTTONS_OK, 
+                message_format = str(ex))
+            msg_dlg.run()
+            msg_dlg.destroy()
+        finally:
+            imp_window.destroy()
+
+
     def preferences(self, source):
         pref_window = PreferencesWindow(self.__window)
         try:
@@ -1076,6 +1222,8 @@ class Program(hildon.Program):
                     self.__model.update_cache_path(new_path)
                 self.__model.set_checkboxes(pref_window.checkboxes_check.get_active())
                 self.check_col.set_visible(self.__model.use_checkboxes)
+                self.refresh_selected_item.set_property("visible", 
+                    not self.__model.use_checkboxes)
                 self.__model.save_config()
         except Exception, ex:
             msg_dlg = gtk.MessageDialog(parent = self.__window, flags = gtk.DIALOG_MODAL,
@@ -1090,7 +1238,7 @@ class Program(hildon.Program):
     def about(self, source):
         dialog = gtk.AboutDialog()
         dialog.set_name("FeedCircuit")
-        dialog.set_version("0.6.8")
+        dialog.set_version("0.6.9")
         dialog.set_website("http://feedcircuit.garage.maemo.org")
         dialog.set_authors([_("George Kibardin")])
         dialog.run()
